@@ -8,18 +8,19 @@ import numbers
 
 import numpy as np
 
-from astropy.nddata import NDData
+from astropy.nddata import NDDataArray
 from astropy.nddata.nduncertainty import StdDevUncertainty, NDUncertainty
 from astropy.io import fits, registry
 from astropy import units as u
 from astropy import log
+from astropy.wcs import WCS
 
 from .utils.collections import CaseInsensitiveOrderedDict
 
 __all__ = ['CCDData']
 
 
-class CCDData(NDData):
+class CCDData(NDDataArray):
 
     """A class describing basic CCD data
 
@@ -35,8 +36,8 @@ class CCDData(NDData):
         make copy the `data` before passing it in if that's the  desired
         behavior.
 
-    uncertainty : `~astropy.nddata.StdDevUncertainty` or `~numpy.ndarray`, optional
-        Uncertainties on the data.
+    uncertainty : `~astropy.nddata.StdDevUncertainty` or `~numpy.ndarray`,
+        optional Uncertainties on the data.
 
     mask : `~numpy.ndarray`, optional
         Mask for the data, given as a boolean Numpy array with a shape
@@ -52,14 +53,9 @@ class CCDData(NDData):
         data, or as a `~astropy.nddata.FlagCollection` instance which has a
         shape matching that of the data.
 
-    wcs : undefined, optional
+    wcs : `~astropy.wcs.WCS` object, optional
         WCS-object containing the world coordinate system for the data.
 
-        .. warning::
-            This is not yet defind because the discussion of how best to
-            represent this class's WCS system generically is still under
-            consideration. For now just leave it as None
-
     meta : `dict`-like object, optional
         Metadata for this object.  "Metadata" here means all information that
         is included with this object but not part of any other attribute
@@ -101,10 +97,46 @@ class CCDData(NDData):
 
     """
     def __init__(self, *args, **kwd):
+        if 'meta' not in kwd:
+            kwd['meta'] = kwd.pop('header', None)
+        if 'header' in kwd:
+            raise ValueError("Can't have both header and meta")
+        
         super(CCDData, self).__init__(*args, **kwd)
         if self.unit is None:
             raise ValueError("Unit for CCDData must be specified")
 
+    @property
+    def data(self):
+        return self._data
+
+    @data.setter
+    def data(self, value):
+        self._data = value
+
+    @property
+    def unit(self):
+        return self._unit
+
+    @unit.setter
+    def unit(self, value):
+        if value is not None:
+            self._unit = u.Unit(value)
+        else:
+            self._unit = None
+
+    @property
+    def shape(self):
+        return self.data.shape
+
+    @property
+    def size(self):
+        return self.data.size
+
+    @property
+    def dtype(self):
+        return self.data.dtype
+
     @property
     def header(self):
         return self._meta
@@ -164,19 +196,36 @@ class CCDData(NDData):
 
         """
         if isinstance(self.header, fits.Header):
-            header = self.header
+            # Copy here so that we can modify the HDU header by adding WCS
+            # information without changing the header of the CCDData object.
+            header = self.header.copy()
         else:
             # Because _insert_in_metadata_fits_safe is written as a method
             # we need to create a dummy CCDData instance to hold the FITS
             # header we are constructing. This probably indicates that
             # _insert_in_metadata_fits_safe should be rewritten in a more
             # sensible way...
-            dummy_ccd = CCDData(data=[1], meta=fits.Header(), unit="adu")
+            dummy_ccd = CCDData([1], meta=fits.Header(), unit="adu")
             for k, v in self.header.items():
                 dummy_ccd._insert_in_metadata_fits_safe(k, v)
             header = dummy_ccd.header
         if self.unit is not u.dimensionless_unscaled:
             header['bunit'] = self.unit.to_string()
+        if self.wcs:
+            # Simply extending the FITS header with the WCS can lead to
+            # duplicates of the WCS keywords; iterating over the WCS
+            # header should be safer.
+            #
+            # Turns out if I had read the io.fits.Header.extend docs more
+            # carefully, I would have realized that the keywords exist to
+            # avoid duplicates and preserve, as much as possible, the
+            # structure of the commentary cards.
+            #
+            # Note that until astropy/astropy#3967 is closed, the extend
+            # will fail if there are comment cards in the WCS header but
+            # not header.
+            wcs_header = self.wcs.to_header()
+            header.extend(wcs_header, useblanks=False, update=True)
         hdu = fits.PrimaryHDU(self.data, header)
         hdulist = fits.HDUList([hdu])
         return hdulist
@@ -216,7 +265,7 @@ class CCDData(NDData):
         else:
             result_uncertainty = None
 
-        result = CCDData(data=result_data, unit=result_unit,
+        result = CCDData(result_data, unit=result_unit,
                          uncertainty=result_uncertainty,
                          meta=self.meta)
         return result
@@ -302,7 +351,9 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, **kwd):
         Name of fits file.
 
     hdu : int, optional
-        FITS extension from which CCDData should be initialized.
+        FITS extension from which CCDData should be initialized.  If zero and
+        and no data in the primary extention, it will search for the first
+        extension with data.
 
     unit : astropy.units.Unit, optional
         Units of the image data. If this argument is provided and there is a
@@ -332,11 +383,20 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, **kwd):
     hdus = fits.open(filename, **kwd)
     hdr = hdus[hdu].header
 
+    # search for the first instance with data if the primary header is empty
+    if hdu == 0 and hdus[hdu].data is None:
+        for i in range(len(hdus)):
+            if hdus.fileinfo(i)['datSpan'] > 0:
+                hdu = i
+                log.info("First HDU with data is exention {0}".format(hdu))
+                break
+
     try:
         fits_unit_string = hdr['bunit']
-        #patch to handle FITS files using ADU for the unit instead of the 
-        #standard version of 'adu'
-        if fits_unit_string.strip().lower()=='adu': fits_unit_string=fits_unit_string.lower()
+        # patch to handle FITS files using ADU for the unit instead of the
+        # standard version of 'adu'
+        if fits_unit_string.strip().lower() == 'adu':
+            fits_unit_string = fits_unit_string.lower()
     except KeyError:
         fits_unit_string = None
 
@@ -346,7 +406,14 @@ def fits_ccddata_reader(filename, hdu=0, unit=None, **kwd):
                                                          fits_unit_string))
 
     use_unit = unit or fits_unit_string
-    ccd_data = CCDData(hdus[hdu].data, meta=hdus[hdu].header, unit=use_unit)
+    # Try constructing a WCS object. This may generate a warning, but never
+    # an error.
+    wcs = WCS(hdr)
+    # Test for success by checking to see if the wcs ctype has a non-empty
+    # value.
+    wcs = wcs if wcs.wcs.ctype[0] else None
+    ccd_data = CCDData(hdus[hdu].data, meta=hdus[hdu].header, unit=use_unit,
+                       wcs=wcs)
     hdus.close()
     return ccd_data
 
