@@ -219,11 +219,16 @@ def subtract_overscan(ccd, overscan=None, overscan_axis=1, fits_section=None,
         else:
             oscan = np.reshape(oscan, (1, oscan.size))
     else:
-        oscan = np.reshape(oscan, oscan.shape + (1,))
+        if overscan_axis == 1:
+            oscan = np.reshape(oscan, oscan.shape + (1,))
+        else:
+            oscan = np.reshape(oscan, (1,) + oscan.shape)
+
+    subtracted = ccd.copy()
 
     # subtract the overscan
-    ccd.data = ccd.data - oscan
-    return ccd
+    subtracted.data = ccd.data - oscan
+    return subtracted
 
 
 @log_to_metadata
