@@ -11,6 +11,7 @@ from astropy.nddata import StdDevUncertainty
 from astropy import units as u
 from astropy.extern import six
 from astropy import log
+from astropy.wcs import WCS
 
 from ..ccddata import CCDData
 from .. import subtract_dark
@@ -66,6 +67,7 @@ def test_initialize_from_fits_with_unit_in_header(tmpdir):
     ccd2 = CCDData.read(filename, unit="photon")
     assert ccd2.unit is u.photon
 
+
 def test_initialize_from_fits_with_ADU_in_header(tmpdir):
     fake_img = np.random.random(size=(100, 100))
     hdu = fits.PrimaryHDU(fake_img)
@@ -77,6 +79,33 @@ def test_initialize_from_fits_with_ADU_in_header(tmpdir):
     assert ccd.unit is u.adu
 
 
+def test_initialize_from_fits_with_data_in_different_extension(tmpdir):
+    fake_img = np.random.random(size=(100, 100))
+    new_hdul = fits.HDUList()
+    hdu1 = fits.PrimaryHDU()
+    hdu2 = fits.ImageHDU(fake_img)
+    hdus = fits.HDUList([hdu1, hdu2])
+    filename = tmpdir.join('afile.fits').strpath
+    hdus.writeto(filename)
+    ccd = CCDData.read(filename, unit='adu')
+    # ccd should pick up the unit adu from the fits header...did it?
+    np.testing.assert_array_equal(ccd.data, fake_img)
+
+
+def test_initialize_from_fits_with_extension(tmpdir):
+    fake_img1 = np.random.random(size=(100, 100))
+    fake_img2 = np.random.random(size=(100, 100))
+    new_hdul = fits.HDUList()
+    hdu0 = fits.PrimaryHDU()
+    hdu1 = fits.ImageHDU(fake_img1)
+    hdu2 = fits.ImageHDU(fake_img2)
+    hdus = fits.HDUList([hdu0, hdu1, hdu2])
+    filename = tmpdir.join('afile.fits').strpath
+    hdus.writeto(filename)
+    ccd = CCDData.read(filename, hdu=2, unit='adu')
+    # ccd should pick up the unit adu from the fits header...did it?
+    np.testing.assert_array_equal(ccd.data, fake_img2)
+
 
 def test_write_unit_to_hdu(ccd_data, tmpdir):
     ccd_unit = ccd_data.unit
@@ -431,7 +460,8 @@ def test_ccddata_with_fits_header_as_meta_works_with_autologging(ccd_data,
 
 
 def test_history_preserved_if_metadata_is_fits_header(tmpdir):
-    hdu = fits.PrimaryHDU()
+    fake_img = np.random.random(size=(100, 100))
+    hdu = fits.PrimaryHDU(fake_img)
     hdu.header['history'] = 'one'
     hdu.header['history'] = 'two'
     hdu.header['history'] = 'three'
@@ -451,3 +481,66 @@ def test_infol_logged_if_unit_in_fits_header(ccd_data, tmpdir):
     with log.log_to_list() as log_list:
         ccd_from_disk = CCDData.read(tmpfile.strpath, unit=explicit_unit_name)
         assert explicit_unit_name in log_list[0].message
+
+
+def test_wcs_attribute(ccd_data, tmpdir):
+    """
+    Check that WCS attribute gets added to header, and that if a CCDData
+    object is created from a FITS file with a header, and the WCS attribute
+    is modified, then the CCDData object is turned back into an hdu, the
+    WCS object overwrites the old WCS information in the header.
+    """
+    tmpfile = tmpdir.join('temp.fits')
+    # This wcs example is taken from the astropy.wcs docs.
+    wcs = WCS(naxis=2)
+    wcs.wcs.crpix = np.array(ccd_data.shape)/2
+    wcs.wcs.cdelt = np.array([-0.066667, 0.066667])
+    wcs.wcs.crval = [0, -90]
+    wcs.wcs.ctype = ["RA---AIR", "DEC--AIR"]
+    wcs.wcs.set_pv([(2, 1, 45.0)])
+    ccd_data.header = ccd_data.to_hdu()[0].header
+    ccd_data.header.extend(wcs.to_header(), useblanks=False)
+    ccd_data.write(tmpfile.strpath)
+    ccd_new = CCDData.read(tmpfile.strpath)
+    original_header_length = len(ccd_new.header)
+    # WCS attribute should be set for ccd_new
+    assert ccd_new.wcs is not None
+    # WCS attribute should be equal to wcs above.
+    assert ccd_new.wcs.wcs == wcs.wcs
+
+    # Converting CCDData object with wcs to an hdu shouldn't
+    # create duplicate wcs-related entries in the header.
+    ccd_new_hdu = ccd_new.to_hdu()[0]
+    assert len(ccd_new_hdu.header) == original_header_length
+
+    # Making a CCDData with WCS (but not WCS in the header) should lead to
+    # WCS information in the header when it is converted to an HDU.
+    ccd_wcs_not_in_header = CCDData(ccd_data.data, wcs=wcs, unit="adu")
+    hdu = ccd_wcs_not_in_header.to_hdu()[0]
+    wcs_header = wcs.to_header()
+    for k in wcs_header.keys():
+        # Skip these keywords if they are in the WCS header because they are
+        # not WCS-specific.
+        if k in ['', 'COMMENT', 'HISTORY']:
+            continue
+        # No keyword from the WCS should be in the header.
+        assert k not in ccd_wcs_not_in_header.header
+        # Every keyword in the WCS should be in the header of the HDU
+        assert hdu.header[k] == wcs_header[k]
+
+    # Now check that if WCS of a CCDData is modified, then the CCDData is
+    # converted to an HDU, the WCS keywords in the header are overwritten
+    # with the appropriate keywords from the header.
+    #
+    # ccd_new has a WCS and WCS keywords in the header, so try modifying
+    # the WCS.
+    ccd_new.wcs.wcs.cdelt *= 2
+    ccd_new_hdu_mod_wcs = ccd_new.to_hdu()[0]
+    assert ccd_new_hdu_mod_wcs.header['CDELT1'] == ccd_new.wcs.wcs.cdelt[0]
+    assert ccd_new_hdu_mod_wcs.header['CDELT2'] == ccd_new.wcs.wcs.cdelt[1]
+
+
+def test_header(ccd_data):
+    a = {'Observer': 'Hubble'}
+    ccd = CCDData(ccd_data, header = a)
+    assert ccd.meta == a
