@@ -2,12 +2,13 @@
 # This module implements the base CCDData class
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
-
 import numpy as np
 from astropy.io import fits
 from astropy.modeling import models
 from astropy.units.quantity import Quantity
 import astropy.units as u
+from astropy.wcs import WCS
+
 
 from astropy.nddata import StdDevUncertainty
 
@@ -72,11 +73,16 @@ def test_create_deviation_keywords_must_have_unit(ccd_data):
 
 
 # tests for overscan
+@pytest.mark.parametrize('data_rectangle', [False, True])
 @pytest.mark.parametrize('median,transpose', [
                          (False, False),
                          (False, True),
                          (True, False), ])
-def test_subtract_overscan(ccd_data, median, transpose):
+def test_subtract_overscan(ccd_data, median, transpose, data_rectangle):
+    # Make data non-square if desired
+    if data_rectangle:
+        ccd_data.data = ccd_data.data[:, :-30]
+
     # create the overscan region
     oscan = 300.
     oscan_region = (slice(None), slice(0, 10))  # indices 0 through 9
@@ -215,6 +221,22 @@ def test_trim_image_no_section(ccd_data):
     np.testing.assert_array_equal(trimmed.data, ccd_data[:, 19:40])
 
 
+def test_trim_with_wcs_alters_wcs(ccd_data):
+    # WCS construction example pulled form astropy.wcs docs
+    wcs = WCS(naxis=2)
+    wcs.wcs.crpix = np.array(ccd_data.shape)/2
+    wcs.wcs.cdelt = np.array([-0.066667, 0.066667])
+    wcs.wcs.crval = [0, -90]
+    wcs.wcs.ctype = ["RA---AIR", "DEC--AIR"]
+    wcs.wcs.set_pv([(2, 1, 45.0)])
+    ccd_wcs = CCDData(ccd_data, wcs=wcs)
+    # The trim below should subtract 10 from the 2nd element of crpix.
+    # (Second element because the FITS convention for index ordering is
+    #  opposite that of python)
+    trimmed = trim_image(ccd_wcs[10:, :])
+    assert trimmed.wcs.wcs.crpix[1] == wcs.wcs.crpix[1] - 10
+
+
 def test_subtract_bias(ccd_data):
     data_avg = ccd_data.data.mean()
     bias_level = 5.0
@@ -253,7 +275,7 @@ def test_subtract_dark(ccd_data, explicit_times, scale, exposure_keyword):
     exposure_unit = u.second
     dark_level = 1.7
     master_dark_data = np.zeros_like(ccd_data.data) + dark_level
-    master_dark = CCDData(data=master_dark_data, unit=u.adu)
+    master_dark = CCDData(master_dark_data, unit=u.adu)
     master_dark.header[exptime_key] = 2 * exptime
     dark_exptime = master_dark.header[exptime_key]
     ccd_data.header[exptime_key] = exptime
@@ -514,3 +536,72 @@ def test__blkavg_larger(ccd_data):
 
     assert b.shape == (10, 10)
     np.testing.assert_almost_equal(b.sum(), 0.25 * a.sum())
+
+
+#test overscan changes
+def test__overscan_schange(ccd_data):
+    old_data = ccd_data.copy()
+    new_data = subtract_overscan(ccd_data, overscan=ccd_data[:,1], overscan_axis=0)
+    assert not np.allclose(old_data.data, new_data.data)
+    np.testing.assert_array_equal(old_data.data, ccd_data.data)
+
+
+def test_create_deviation_does_not_change_input(ccd_data):
+    original = ccd_data.copy()
+    ccd = create_deviation(ccd_data, gain=5 * u.electron / u.adu, readnoise=10 * u.electron)
+    np.testing.assert_array_equal(original.data, ccd_data.data)
+    assert original.unit == ccd_data.unit
+
+def test_cosmicray_median_does_not_change_input(ccd_data):
+	original = ccd_data.copy()
+	error = np.zeros_like(ccd_data)
+	ccd = cosmicray_median(ccd_data,error_image = error, thresh = 5, mbox = 11, gbox = 0, rbox = 0)
+	np.testing.assert_array_equal(original.data,ccd_data.data)
+	assert original.unit == ccd_data.unit
+
+def test_cosmicray_lacosmic_does_not_change_input(ccd_data):
+	original = ccd_data.copy()
+	error = np.zeros_like(ccd_data)
+	ccd = cosmicray_lacosmic(ccd_data, error_image = error, thresh = 5, fthresh = 5, gthresh = 1.5, b_factor = 2, mbox = 5, min_limit = 0.01, gbox = 0, rbox = 0)
+	np.testing.assert_array_equal(original.data, ccd_data.data)
+	assert original.unit == ccd_data.unit
+
+def test_flat_correct_does_not_change_input(ccd_data):
+	original = ccd_data.copy()
+	flat = CCDData(np.zeros_like(ccd_data), unit = ccd_data.unit)
+	ccd = flat_correct(ccd_data,flat=flat)
+	np.testing.assert_array_equal(original.data, ccd_data.data)
+	assert original.unit == ccd_data.unit
+
+def test_gain_correct_does_not_change_input(ccd_data):
+    original = ccd_data.copy()
+    ccd = gain_correct(ccd_data, gain = 1, gain_unit = ccd_data.unit)
+    np.testing.assert_array_equal(original.data, ccd_data.data)
+    assert original.unit == ccd_data.unit
+
+def test_subtract_bias_does_not_change_input(ccd_data):
+    original = ccd_data.copy()
+    master_frame = CCDData(np.zeros_like(ccd_data),unit = ccd_data.unit)
+    ccd = subtract_bias(ccd_data, master = master_frame)
+    np.testing.assert_array_equal(original.data, ccd_data.data)
+    assert original.unit == ccd_data.unit
+
+def test_trim_image_does_not_change_input(ccd_data):
+    original = ccd_data.copy()
+    ccd = trim_image(ccd_data, fits_section = None)
+    np.testing.assert_array_equal(original.data, ccd_data.data)
+    assert original.unit == ccd_data.unit
+
+def test_rebin_does_not_change_input(ccd_data):
+    original = ccd_data.copy()
+    ccd = rebin(ccd_data, (20,20))
+    np.testing.assert_array_equal(original.data, ccd_data.data)
+    assert original.unit == ccd_data.unit
+
+def test_transform_image_does_not_change_input(ccd_data):
+    original = ccd_data.copy()
+    ccd = transform_image(ccd_data,np.sqrt)
+    np.testing.assert_array_equal(original.data, ccd_data)
+    assert original.unit == ccd_data.unit
+
+
