@@ -66,6 +66,12 @@ def test_combiner_dtype(ccd_data):
     ccd_list = [ccd_data, ccd_data, ccd_data]
     c = Combiner(ccd_list, dtype = np.float32)
     assert c.data_arr.dtype == np.float32
+    avg = c.average_combine()
+    # dtype of average should match input dtype
+    assert avg.dtype == c.dtype
+    med = c.median_combine()
+    # dtype of median should match dtype of input
+    assert med.dtype == c.dtype
 
 
 #test mask is created from ccd.data
@@ -231,8 +237,8 @@ def test_combiner_with_scaling(ccd_data):
     assert avg_ccd.shape == ccd_data.shape
     median_ccd = combiner.median_combine()
     # Does median also scale to the correct value?
-    np.testing.assert_almost_equal(np.ma.median(median_ccd),
-                                   np.ma.median(ccd_data.data))
+    np.testing.assert_almost_equal(np.median(median_ccd),
+                                   np.median(ccd_data.data))
 
     # Set the scaling manually...
     combiner.scaling = [scale_by_mean(combiner.data_arr[i]) for i in range(3)]
